/*******************************************************************
  uLan Utilities Library - C library of basic reusable constructions

  ul_hastabcust.h	- custom hash table implementation

  (C) Copyright 2009 by Pavel Pisa - Originator

  The uLan utilities library can be used, copied and modified under
  next licenses
    - GPL - GNU General Public License
    - LGPL - GNU Lesser General Public License
    - MPL - Mozilla Public License
    - and other licenses added by project originators
  Code can be modified and re-distributed under any combination
  of the above listed licenses. If contributor does not agree with
  some of the licenses, he/she can delete appropriate line.
  Warning, if you delete all lines, you are not allowed to
  distribute source code and/or binaries utilizing code.
  
  See files COPYING and README for details.

 *******************************************************************/

#ifndef _UL_HASHTABCUST_H
#define _UL_HASHTABCUST_H

#include "ul_utdefs.h"
#include "ul_hashtab.h"
#include "ul_gavlcust.h"

#ifdef __cplusplus
extern "C" {
#endif

#define UL_HASTAB_CUST_NODE_INT_IMP(cust_prefix, cust_table_t, cust_item_t, cust_key_t,\
		cust_table_field, cust_item_node, cust_item_key, cust_cmp_fnc, cust_hash_fnc, \
		cust_hash_get_fnc, cust_hash_prep_fnc, cust_sizestep) \
GAVL_CUST_NODE_INT_DEC_SCOPE(static, cust_prefix##_privtree, ul_hashtab_treeroot_t, cust_item_t, cust_key_t,\
	treeroot, cust_item_node.treenode, cust_item_key, cust_cmp_fnc) \
GAVL_CUST_NODE_INT_IMP(cust_prefix##_privtree, ul_hashtab_treeroot_t, cust_item_t, cust_key_t,\
	treeroot, cust_item_node.treenode, cust_item_key, cust_cmp_fnc) \
\
static inline \
ul_hashtab_treeroot_t *cust_prefix##_privtree_tree4item(cust_table_t *table, const cust_item_t *item)\
{\
  return !table->cust_table_field.hashmask? &(table->cust_table_field.treeroots.treesingle):\
    &(table->cust_table_field.treeroots.treetable[cust_hash_get_fnc(item)&table->cust_table_field.hashmask]);\
} \
\
static inline \
const ul_hashtab_treeroot_t *cust_prefix##_privtree_tree4key(const cust_table_t *table, cust_key_t const *key)\
{\
  return !table->cust_table_field.hashmask? &(table->cust_table_field.treeroots.treesingle):\
    &(table->cust_table_field.treeroots.treetable[cust_hash_fnc(key)&table->cust_table_field.hashmask]);\
} \
\
static inline \
void cust_prefix##_privdummy(cust_table_t *table) \
{/*used to suppress warning about unused static function generated by GAVL*/\
  cust3_privtree_init_root_field(&(table->cust_table_field.treeroots.treesingle));\
} \
\
static \
int cust_prefix##_privinternal_insert(cust_table_t *table, cust_item_t *item)\
{\
  return cust_prefix##_privtree_insert(cust_prefix##_privtree_tree4item(table, item), item);\
} \
\
static \
int cust_prefix##_privresize_insert(ul_hashtab_cust_table_field_t *table_field, gavl_node_t *treenode) UL_ATTR_REENTRANT\
{\
  return cust_prefix##_privinternal_insert(UL_CONTAINEROF(table_field, cust_table_t, cust_table_field),\
                                           cust_prefix##_privtree_node2item(NULL,treenode));\
} \
\
int cust_prefix##_resize_table(cust_table_t *table, ul_hashtab_hashval_t newsize)\
{\
  return ul_hashtab_resize_primitive(&table->cust_table_field, newsize, cust_prefix##_privresize_insert);\
} \
\
int cust_prefix##_insert(cust_table_t *table, cust_item_t *item)\
{\
  int res;\
  if((cust_sizestep)!=ul_hashtab_sizestep_null) \
    if((table->cust_table_field.sizestep<ul_hashtab_sizestep_max(cust_sizestep))&& \
       (table->cust_table_field.count>=(cust_sizestep)[table->cust_table_field.sizestep].toexpand)) {\
      table->cust_table_field.sizestep++; \
      cust_prefix##_resize_table(table, (cust_sizestep)[table->cust_table_field.sizestep].size);\
  } \
  cust_hash_prep_fnc(item);\
  res=cust_prefix##_privinternal_insert(table, item);\
  if(res>0) \
    table->cust_table_field.count++; \
  return res; \
} \
\
int cust_prefix##_delete(cust_table_t *table, cust_item_t *item)\
{\
  int res;\
  res=cust_prefix##_privtree_delete(cust_prefix##_privtree_tree4item(table, item), item);\
  if(res>0) { \
    table->cust_table_field.count--; \
    if(((cust_sizestep)!=ul_hashtab_sizestep_null)&&table->cust_table_field.sizestep) {\
      if(table->cust_table_field.sizestep>ul_hashtab_sizestep_max(cust_sizestep)) \
        table->cust_table_field.sizestep=ul_hashtab_sizestep_max(cust_sizestep); \
      if(table->cust_table_field.count<=(cust_sizestep)[table->cust_table_field.sizestep].toshrink) {\
        table->cust_table_field.sizestep--; \
        cust_prefix##_resize_table(table, (cust_sizestep)[table->cust_table_field.sizestep].size);\
      } \
    } \
  }\
  return res; \
} \
\
cust_item_t *cust_prefix##_delete_key(cust_table_t *table, cust_key_t const *key) \
{\
  gavl_node_t *node; \
  const ul_hashtab_treeroot_t *treeroot; \
  treeroot=cust_prefix##_privtree_tree4key(table, key); \
  if(cust_prefix##_privtree_search_node(treeroot, key, &node)) \
    return NULL; \
  cust_prefix##_privtree_delete_node((ul_hashtab_treeroot_t *)treeroot, node); \
  table->cust_table_field.count--; \
  if(((cust_sizestep)!=ul_hashtab_sizestep_null)&&table->cust_table_field.sizestep) {\
    if(table->cust_table_field.sizestep>ul_hashtab_sizestep_max(cust_sizestep)) \
      table->cust_table_field.sizestep=ul_hashtab_sizestep_max(cust_sizestep); \
    if(table->cust_table_field.count<=(cust_sizestep)[table->cust_table_field.sizestep].toshrink) {\
      table->cust_table_field.sizestep--; \
      cust_prefix##_resize_table(table, (cust_sizestep)[table->cust_table_field.sizestep].size);\
    } \
  } \
  return cust_prefix##_privtree_node2item(NULL,node); \
} \
\
int cust_prefix##_search_node(const cust_table_t *table, cust_key_t const *key, typeof(((cust_item_t*)0)->cust_item_node) **nodep) \
{\
  gavl_node_t *node;\
  if(cust_prefix##_privtree_search_node(cust_prefix##_privtree_tree4key(table, key), key, &node)) { \
    *nodep=NULL; \
    return -1; \
  } \
  *nodep=&(UL_CONTAINEROF(node, cust_item_t, cust_item_node.treenode)->cust_item_node); \
  return 0; \
} \
\
cust_item_t *cust_prefix##_find(const cust_table_t *table, cust_key_t const *key) \
{\
  return cust_prefix##_privtree_find(cust_prefix##_privtree_tree4key(table, key), key); \
} \
\
int cust_prefix##_find_it(cust_table_t *container, cust_key_t const *key, cust_prefix##_it_t *it) \
{\
  it->container=container; \
  it->ptreeroot=(ul_hashtab_treeroot_t*)cust_prefix##_privtree_tree4key(it->container, key); \
  it->item=cust_prefix##_privtree_find(it->ptreeroot, key); \
  return it->item!=NULL?1:0; \
}\
\
void cust_prefix##_delete_it(cust_prefix##_it_t *it)\
{\
  cust_item_t *item;\
  ul_hashtab_treeroot_t *treeroot; \
  if(!(item=it->item)) return;\
  treeroot=it->ptreeroot; \
  cust_prefix##_next_it(it);\
  if(cust_prefix##_privtree_delete(treeroot,item)>=0)\
    it->container->cust_table_field.count--; \
}

#ifdef __cplusplus
} /* extern "C"*/
#endif

#endif /* _UL_HASHTABCUST_H */
